// Copyright 2021 Contributors to the Parsec project.
// SPDX-License-Identifier: Apache-2.0
#[path = "common/mod.rs"]
mod common;

mod abstraction_tests;
mod attributes_tests;
mod constants_tests;
mod context_tests;
mod handles_tests;
mod interface_types_tests;
mod structures_tests;
mod tcti_ldr_tests;
mod utils_tests;
